package com.cyrilmottier.android.tutorial5;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

/**
 * View allowing to trash a text file by simply dragging it upon the trash.
 * 
 * @author Cyril Mottier
 * @version 0.5
 */
public class TrashView extends View {

    private static final boolean LOG_ENABLED = true;
    private static final String LOG_TAG = "TrashView";
    private final int ICON_SIZE = 100;

    /**
     * When dragging the file to the trash, it defines the percentage of
     * overlapping surface beyond which the file is considered as trashed.
     */
    private static final float SURFACE_THRESHOLD = 0.6f;

    /**
     * Bitmaps for all images displayed in the view.
     */
    private Bitmap mEmptyTrash;
    private Bitmap mFullTrash;
    private Bitmap mTextFile;

    private boolean mIsTrashFull;

    private int mIconsSize;

    private float mFileStartX;
    private float mFileStartY;

    private float mPreviousFileX;
    private float mPreviousFileY;
    private float mFileX;
    private float mFileY;
    private float mTrashX;
    private float mTrashY;

    private float mDeltaX;
    private float mDeltaY;

    public TrashView(Context context) {
        super(context);
        mIconsSize = ICON_SIZE;
        mEmptyTrash = prepareBitmap(getResources().getDrawable(R.drawable.empty_trash), mIconsSize,
                mIconsSize);
        mFullTrash = prepareBitmap(getResources().getDrawable(R.drawable.full_trash), mIconsSize,
                mIconsSize);
        mTextFile = prepareBitmap(getResources().getDrawable(R.drawable.txt_file), mIconsSize,
                mIconsSize);
        resetTrashView();
    }

    public void resetTrashView() {
        mIsTrashFull = false;
        mFileStartX = 10;
        mFileStartY = 20;
        mFileX = mFileStartX;
        mFileY = mFileStartY;
        mTrashX = 200;
        mTrashY = 20;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        if (mIsTrashFull) {
            canvas.drawBitmap(mFullTrash, mTrashX, mTrashY, null);
        } else {
            canvas.drawBitmap(mEmptyTrash, mTrashX, mTrashY, null);
            canvas.drawBitmap(mTextFile, mFileX, mFileY, null);
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {

        final int action = event.getAction();
        final float x = event.getX();
        final float y = event.getY();

        switch (action) {
            case MotionEvent.ACTION_DOWN:
                if (!mIsTrashFull && x >= mFileX && x <= mFileX + mIconsSize && y >= mFileY
                        && y <= mFileY + mIconsSize) {
                    log("Start moving text file");
                    mDeltaX = x - mFileX;
                    mDeltaY = y - mFileY;
                    mPreviousFileX = mFileX;
                    mPreviousFileY = mFileY;
                    return true;
                }
                break;

            case MotionEvent.ACTION_MOVE:
                mFileX = x - mDeltaX;
                mFileY = y - mDeltaY;
                optimizedInvalidate();
                return true;

            case MotionEvent.ACTION_UP:
                log("Stop moving text file");
                if (intersect(mFileX, mIconsSize, mTrashX, mIconsSize)
                        * intersect(mFileY, mIconsSize, mTrashY, mIconsSize) > mIconsSize
                        * mIconsSize * SURFACE_THRESHOLD) {
                    mIsTrashFull = true;
                    log("File trashed");
                } else {
                    mFileX = mFileStartX;
                    mFileY = mFileStartY;
                }
                invalidate();
                return true;
        }
        return super.onTouchEvent(event);
    }

    private void optimizedInvalidate() {
        final int iconSize = mIconsSize;
        final float fileX = mFileX;
        final float fileY = mFileY;
        /*
         * l and t must be the minimum values
         */
        final int l = (int)Math.min(fileX, mPreviousFileX);
        final int t = (int)Math.min(fileY, mPreviousFileY);
        /*
         * b and r must be the maximum values
         */
        final int b = (int)Math.max(fileX + iconSize, mPreviousFileX + iconSize);
        final int r = (int)Math.max(fileY + iconSize, mPreviousFileY + iconSize);
        mPreviousFileX = fileX;
        mPreviousFileY = fileY;
        invalidate(l, t, b, r);
    }

    /**
     * Copies the content of the Drawable <em>drawable</em> to a newly allocated
     * Bitmap of width <em>width</em> and height <em>height</em>.
     * 
     * @param drawable The Drawable to copy to a Bitmap.
     * @param height Height of the Bitmap to create.
     * @param width Width of the Bitmap to create.
     * @return A Bitmap displaying <em>drawable</em> in the size <em>width</em>
     *         and <em>height</em>.
     */
    private static Bitmap prepareBitmap(Drawable drawable, int width, int height) {
        Bitmap bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
        drawable.setBounds(0, 0, width, height);
        Canvas canvas = new Canvas(bitmap);
        drawable.draw(canvas);
        return bitmap;
    }

    /**
     * Computes the intersection length between two segments defined with an
     * origin x and a positive length l.
     * 
     * @param x1 Origin of the first segment
     * @param l1 Length of the first segment
     * @param x2 Origin of the second segment
     * @param l2 Length of the second segment
     * @return The length of the intersection segment between (x1,l1) and (x2,
     *         l2)
     */
    private static double intersect(float x1, int l1, float x2, int l2) {
        double result;
        if (x1 < x2) {
            result = Math.max(0, x1 + l1 - x2);
        } else {
            result = Math.max(0, x2 + l2 - x1);
        }
        return result;
    }

    /**
     * Utility that displays debug messages.
     * 
     * @param log The message to display
     */
    private static void log(String log) {
        if (LOG_ENABLED) {
            Log.d(LOG_TAG, log);
        }
    }
}
